
// Adjusted for MinGW
// Added bypass dialog for forcesetup = 0
// Added extra gamers, maps, saved to menu
// Added extra multiplay to menu
//

#ifndef RENDERTYPEWIN
#error Only for Windows
#endif

#include "duke3d.h"
#include "sounds.h"

#include "build.h"
#include "winlayer.h"
#include "compat.h"

#include "grpscan.h"

#include "config.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#define _WIN32_IE 0x0400
#include <commctrl.h>
#include <stdio.h>

#include "startwin.game.h"

#define TAB_VIDEO 0
#define TAB_SOUND 1
#define TAB_SAVE 2
#define TAB_GAME 3
#define TAB_MAPS 4
#define TAB_MULT 5
#define TAB_MESG 6

static struct audioenumdrv *wavedevs = NULL;

static struct
{
	int fullscreen;
	int widescreen;
	int brightness;
	int stereo;
	int adultmode;
	int highdetail;
	int autoaiming;
	int invertmouse;
	int tilting;
	int messages;
	int xdim, ydim, bpp;
	int forcesetup;
	int usemouse, usejoy;
	char selectedgrp[BMAX_PATH+1];
	int game;
    int ambience;
    int duketalk;
} settings;

static HWND startupdlg = NULL;
static HWND pages[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
static int done = -1, mode = TAB_VIDEO;

//-------------------------- startup menu settings -------------------------

static CACHE1D_FIND_REC *finddirs, *findfiles, *finddirshigh, *findfileshigh;

char  RootFolder[128];
char  MapsFolder[80];
char  MusicFolder[80];
char  GameFolder[80];

char  LastMapsFolder[80];
char  LastMusicFolder[80];
char  LastGameFolder[80];

char  MenuMap[80];
char  MenuMusic[80];
char  MenuGame[80];
char  GameMap[40];

short UserSavenum = -1;
short iLevelNum = -1;
short iSkill = 3;

char  IPAddress[32];
short SelectMulti = -1;
short HostMulti = -1;
short NumPlayerArg = 2;
short DedicatedArg = -1;
short FakePlayer = -1;
short NumBots = 1;
short LCD;

char  UserSaves[10][80];
short NumUserSaves;

extern char PlayerNameArg[32];
extern char IPAddressArg[8][40];
extern short LcdMon;

long PreCache=1, TexCache=0, Tiles=1, Model=1, TexComp=0;

int ivoices[8] = {1,2,4,8,12,16,24,32};
int isample[7] = {8000, 11025, 16000, 22050, 32000, 44100, 48000};

char *Player[8] = {"1", "2","3","4","5","6","7","8"};
char *Voices[8] = {"1","2","4","8","12","16","24","32"};
char *Sample[7] = {"8000", "11025", "16000", "22050", "32000", "44100", "48000"};

char *defgames[44] = {"E1L1.map", "E1L2.map", "E1L3.map", "E1L4.map", "E1L5.map", "E1L6.map",
                      "E1L7.map", "E1L8.map", "E1L9.map", "E1L10.map",  "E1L11.map",
                      "E2L1.map", "E2L2.map", "E2L3.map", "E2L4.map", "E2L5.map", "E2L6.map",
                      "E2L7.map", "E2L8.map", "E2L9.map", "E2L10.map",  "E2L11.map",
                      "E3L1.map", "E3L2.map", "E3L3.map", "E3L4.map", "E3L5.map", "E3L6.map",
                      "E3L7.map", "E3L8.map", "E3L9.map", "E3L10.map",  "E3L11.map",
                      "E4L1.map", "E4L2.map", "E4L3.map", "E4L4.map", "E4L5.map", "E4L6.map",
                      "E4L7.map", "E4L8.map", "E4L9.map", "E4L10.map",  "E4L11.map"};

//--------------------------------------------------------------------------

static void PopulateForm(int pgs)
{
	char xbuf[128+BMAX_PATH];
	HWND hwnd;
	int i, j, k;

    GetCurrentDirectory(128, RootFolder);       // _getcwd
    strlwr(RootFolder);

	if (pgs & (1<<TAB_VIDEO))
	{
		int mode;

        hwnd = GetDlgItem(pages[TAB_VIDEO], IDBRIGHT);
        SendMessage(hwnd, TBM_SETRANGE, FALSE, MAKELONG(0, 63));
        SendMessage(hwnd, TBM_SETPOS, TRUE, settings.brightness);
        SendMessage(hwnd, TBM_SETTHUMBLENGTH, 110, 0);

		hwnd = GetDlgItem(pages[TAB_VIDEO], IDVIDMODE);

		mode = checkvideomode(&settings.xdim, &settings.ydim, settings.bpp, settings.fullscreen, 1);
		if (mode < 0)
		{
			int cd[] = { 32, 24, 16, 15, 8, 0 };

			for (i=0; cd[i]; )
			{   if (cd[i] >= settings.bpp)
			    i++;
			else
			    break;
			}
			for ( ; cd[i]; i++)
			{
				mode = checkvideomode(&settings.xdim, &settings.ydim, cd[i], settings.fullscreen, 1);
				if (mode < 0) continue;
				settings.bpp = cd[i];
				break;
			}
		}

		Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDFULLSCREEN), (settings.fullscreen ? BST_CHECKED : BST_UNCHECKED));
		Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDWIDESCREEN), (settings.widescreen ? BST_CHECKED : BST_UNCHECKED));

	    if (LCD)
            Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDMONLCD), TRUE);
	    else
	        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDMONCRT), TRUE);

        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDPCACHE), (PreCache ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDTCACHE), (TexCache ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDHTILES), (Tiles ?    BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDHMODEL), (Model ?    BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDCOMPRN), (TexComp ? BST_CHECKED : BST_UNCHECKED));

		ComboBox_ResetContent(hwnd);
		for (i=0; i<validmodecnt; i++)
		{
			if (validmode[i].fs != settings.fullscreen) continue;

			// all modes get added to the 3D mode list
			Bsprintf(xbuf, "%ld x %ld %dbpp", validmode[i].xdim, validmode[i].ydim, validmode[i].bpp);
			j = ComboBox_AddString(hwnd, xbuf);
			ComboBox_SetItemData(hwnd, j, i);
			if (i == mode) ComboBox_SetCurSel(hwnd, j);
		}

	    Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDALWAYSSHOW), (settings.forcesetup ? BST_CHECKED : BST_UNCHECKED));
	}

	if (pgs & (1<<TAB_SOUND))
	{
        PopulateSound();
        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSTEREO),(settings.stereo ?   BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDAMBENC),(settings.ambience ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDDUKETK),(settings.duketalk ? BST_CHECKED : BST_UNCHECKED));
    }

  	if (pgs & (1<<TAB_SAVE))
	{
		hwnd = GetDlgItem(pages[TAB_SAVE], IDSDATA);

        for (i=0; i<NumUserSaves; i++)
            j = ListBox_AddString(hwnd, UserSaves[i]);

        ListBox_SetCurSel(hwnd, -1);
        UserSavenum = -1;
        if (shareware)
            ListBox_Enable(hwnd, 0);

        hwnd = GetDlgItem(pages[TAB_SAVE], IDSKILL);
        ListBox_AddString(hwnd, "PIECE OF CAKE");
        ListBox_AddString(hwnd, "LET'S ROCK");
        ListBox_AddString(hwnd, "COME GET SOME");
        ListBox_AddString(hwnd, "DAMN I'M GOOD");
        iSkill = 3;
        ListBox_SetCurSel(hwnd, 2);

        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSADULT), (settings.adultmode ?  BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSDETAL), (settings.highdetail ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSATAIM), (settings.autoaiming ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSMOUSE), (settings.invertmouse ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSTILTS), (settings.tilting ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSMESGS), (settings.messages ? BST_CHECKED : BST_UNCHECKED));
	}

	if (pgs & (1<<TAB_GAME))
	{
	    hwnd = GetDlgItem(pages[TAB_GAME], IDGDIRC);

	    clearfilenames();
        getdirs(RootFolder);
        i = 0;
        k = -1;
        while (finddirs)
        {
          Bsprintf(xbuf,"%s",finddirs->name);
          ComboBox_AddString(hwnd, xbuf);
          strlwr(xbuf);
          j = Bstrncasecmp(xbuf, "games", 5);
          if (j == 0 && k == -1)
          {
              k = i;
              strcpy(GameFolder, xbuf);
          }
          i++;
          finddirs = finddirs->next;
        }
        ComboBox_SetCurSel(hwnd, k);
        ComboBox_GetLBText(hwnd, k, GameFolder);
        strlwr(GameFolder);
        if (k == 0)
            GameFolder[0] = 0;
        GetGameFiles(GameFolder);
        MenuGame[0] = 0;

        SetDefGames();
        GameMap[0] = 0;
   	}

	if (pgs & (1<<TAB_MAPS))
	{
	    hwnd = GetDlgItem(pages[TAB_MAPS], IDMDIRC);
        getdirs(RootFolder);
        i = 0;
        k = -1;
        while (finddirs)
        {
          Bsprintf(xbuf,"%s",finddirs->name);
          ComboBox_AddString(hwnd, xbuf);
          strlwr(xbuf);
          j = Bstrncasecmp(xbuf, "maps", 4);
          if (j == 0 && k == -1)
          {
              k = i;
              strcpy(MapsFolder, xbuf);
          }
          i++;
          finddirs = finddirs->next;
        }
        ComboBox_SetCurSel(hwnd, k);
        ComboBox_GetLBText(hwnd, k, MapsFolder);
        strlwr(MapsFolder);
        if (k == 0)
            MapsFolder[0] = 0;
        GetMapFiles(MapsFolder);
        MenuMap[0] = 0;

	    hwnd = GetDlgItem(pages[TAB_MAPS], IDMFOLD);
        getdirs(RootFolder);
        i = 0;
        k = -1;
        while (finddirs)
        {
          Bsprintf(xbuf,"%s",finddirs->name);
          ComboBox_AddString(hwnd, xbuf);
          strlwr(xbuf);
          j = Bstrncasecmp(xbuf, "music", 5);
          if (j == 0 && k == -1)
          {
              k = i;
              strcpy(MusicFolder, xbuf);
          }
          i++;
          finddirs = finddirs->next;
        }
        ComboBox_SetCurSel(hwnd, k);
        ComboBox_GetLBText(hwnd, k, MusicFolder);
        strlwr(MusicFolder);
        if (k == 0)
            MusicFolder[0] = 0;
        GetMidiFiles(MusicFolder);
        MenuMusic[0] = 0;
	}

  	if (pgs & (1<<TAB_MULT))
	{
		hwnd = GetDlgItem(pages[TAB_MULT], IDUNAME);
		Edit_SetText(hwnd, PlayerNameArg);
        if (shareware)
            Edit_Enable(hwnd, 0);

		hwnd = GetDlgItem(pages[TAB_MULT], IDUIPAD);
		Edit_SetText(hwnd, IPAddressArg[0]);
        if (shareware)
            Edit_Enable(hwnd, 0);

		hwnd = GetDlgItem(pages[TAB_MULT], IDUIPPS);
        for (i=0; i<8; i++)
        {
            if (strlen(IPAddressArg[i]) < 8)
                break;
            ComboBox_AddString(hwnd, IPAddressArg[i]);
        }
        ComboBox_SetCurSel(hwnd, 0);
        IPAddress[0] = 0;
        if (shareware)
            ComboBox_Enable(hwnd, 0);

        hwnd = GetDlgItem(pages[TAB_MULT], IDUMASTER);
	    Button_SetCheck(hwnd, FALSE);
        if (shareware)
            Button_Enable(hwnd, 0);

		hwnd = GetDlgItem(pages[TAB_MULT], IDUNUMPLS);
        for (i=1; i<8; i++)
        {
            ComboBox_AddString(hwnd, Player[i]);
        }
        ComboBox_SetCurSel(hwnd, 0);
        ComboBox_Enable(hwnd, 0);
        NumPlayerArg = 2;
        if (shareware)
            ComboBox_Enable(hwnd, 0);

        hwnd = GetDlgItem(pages[TAB_MULT], IDUFAKEMP);
	    Button_SetCheck(hwnd, FALSE);
        if (shareware)
            Button_Enable(hwnd, 0);

		hwnd = GetDlgItem(pages[TAB_MULT], IDUNUMBOT);
        for (i=0; i<7; i++)
        {
            ComboBox_AddString(hwnd, Player[i]);
        }
        ComboBox_SetCurSel(hwnd, 0);
        ComboBox_Enable(hwnd, 0);
        NumBots = 1;
        if (shareware)
            ComboBox_Enable(hwnd, 0);

        hwnd = GetDlgItem(pages[TAB_MULT], IDUSELECT);
	    Button_SetCheck(hwnd, FALSE);
        if (shareware)
            Button_Enable(hwnd, 0);

	    SelectMulti = -1;
	    HostMulti = -1;
        FakePlayer = -1;
	}
}

static void ResetForm(int pgs)
{
	HWND hwnd;

    if (pgs == 99)
    {
        hwnd = GetDlgItem(pages[TAB_GAME], IDGMAPS);
	    ListBox_ResetContent(hwnd);
	    ListBox_SetCurSel(hwnd, -1);
	    MenuMap[0] = 0;
	    GameMap[0] = 0;
	    iLevelNum = -1;
	    return;
    }

	if (pgs & (1<<TAB_VIDEO))
	{
        PreCache = useprecache;
        TexCache = glusetexcache;
        Tiles = usehightile;
        Model = usemodels;
        TexComp = glusetexcompr;

        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDPCACHE), (PreCache ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDTCACHE), (TexCache ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDHTILES), (Tiles ?    BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDHMODEL), (Model ?    BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDCOMPRN), (TexComp ?  BST_CHECKED : BST_UNCHECKED));

        LCD = LcdMon;
	    if (LCD)
            Button_SetCheck(GetDlgItem(pages[TAB_VIDEO], IDMONLCD), TRUE);
	    else
	        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDMONCRT), TRUE);
    }

	if (pgs & (1<<TAB_SOUND))
	{
	    settings.stereo = ReverseStereo;

        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSTEREO),(settings.stereo ? BST_CHECKED : BST_UNCHECKED));

        settings.ambience = AmbienceToggle;
        settings.duketalk = VoiceToggle;

        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDAMBENC),(settings.ambience ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDDUKETK),(settings.duketalk ? BST_CHECKED : BST_UNCHECKED));
    }

	if (pgs & (1<<TAB_SAVE))
	{
		hwnd = GetDlgItem(pages[TAB_SAVE], IDSDATA);
        ListBox_SetCurSel(hwnd, -1);
        UserSavenum = -1;

	    settings.adultmode   = !ud.lockout;
	    settings.highdetail  = ud.detail;
	    settings.autoaiming  = AutoAim;
	    settings.invertmouse = ud.mouseflip;
	    settings.tilting     = ud.screen_tilting;
	    settings.messages    = ud.fta_on;

        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSADULT), (settings.adultmode ?  BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSDETAL), (settings.highdetail ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSATAIM), (settings.autoaiming ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSMOUSE), (settings.invertmouse ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSTILTS), (settings.tilting ? BST_CHECKED : BST_UNCHECKED));
        Button_SetCheck(GetDlgItem(pages[TAB_SAVE], IDSMESGS), (settings.messages ? BST_CHECKED : BST_UNCHECKED));
	}

	if (pgs & (1<<TAB_GAME))
	{
		hwnd = GetDlgItem(pages[TAB_GAME], IDGDATA);
        ListBox_SetCurSel(hwnd, -1);
	    MenuGame[0] = 0;
        SetDefGames();
	    GameMap[0] = 0;
	}

	if (pgs & (1<<TAB_MAPS))
	{
		hwnd = GetDlgItem(pages[TAB_MAPS], IDMDATA);
        ListBox_SetCurSel(hwnd, -1);
        MenuMap[0] = 0;

		hwnd = GetDlgItem(pages[TAB_MAPS], IDMUSIC);
        ListBox_SetCurSel(hwnd, -1);
        MenuMusic[0] = 0;
	}

	if (pgs & (1<<TAB_MULT))
	{
        Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUMASTER), FALSE);
        Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUSELECT), FALSE);
        Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUHOST), FALSE);
        Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUJOIN), FALSE);
        Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUFAKEMP), FALSE);
	    SelectMulti = -1;
	    HostMulti = -1;
	    NumBots = 1;
	    FakePlayer = -1;
	}
}

static INT_PTR CALLBACK VideoPageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    short i;
    HWND Hh;

	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDFULLSCREEN:
				    settings.fullscreen = IsDlgButtonChecked(hwndDlg, IDFULLSCREEN) == BST_CHECKED;
					return TRUE;
				case IDVIDMODE:
					if (HIWORD(wParam) == CBN_SELCHANGE)
					{
						i = ComboBox_GetCurSel((HWND)lParam);
						if (i != CB_ERR)
						    i = ComboBox_GetItemData((HWND)lParam, i);
						if (i != CB_ERR)
						{
							settings.xdim = validmode[i].xdim;
							settings.ydim = validmode[i].ydim;
							settings.bpp  = validmode[i].bpp;
						}
					}
					return TRUE;
				case IDBRIGHT:
                     return TRUE;
				case IDWIDESCREEN:
 				     settings.widescreen = IsDlgButtonChecked(hwndDlg, IDWIDESCREEN) == BST_CHECKED;
					 return TRUE;
				case IDPCACHE:
					 PreCache = IsDlgButtonChecked(hwndDlg, IDPCACHE) == BST_CHECKED;
					 return TRUE;
				case IDTCACHE:
					 TexCache = IsDlgButtonChecked(hwndDlg, IDTCACHE) == BST_CHECKED;
					 return TRUE;
				case IDHTILES:
					 Tiles = IsDlgButtonChecked(hwndDlg, IDHTILES) == BST_CHECKED;
					 return TRUE;
				case IDHMODEL:
					 Model = IsDlgButtonChecked(hwndDlg, IDHMODEL) == BST_CHECKED;
					 return TRUE;
				case IDCOMPRN:
					 TexComp = IsDlgButtonChecked(hwndDlg, IDCOMPRN) == BST_CHECKED;
					 return TRUE;
				case IDMONLCD:
					 LCD = 1;
					 return TRUE;
				case IDMONCRT:
					 LCD = 0;
					 return TRUE;
				case IDALWAYSSHOW:
					 settings.forcesetup = IsDlgButtonChecked(hwndDlg, IDALWAYSSHOW) == BST_CHECKED;
					 return TRUE;
				default: break;
			}
			break;
		default: break;
	}
	return FALSE;
}

static INT_PTR CALLBACK SoundPageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    short i;
    HWND Hh;

	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDSOUNDENABLE:
					 SoundToggle = IsDlgButtonChecked(hwndDlg, IDSOUNDENABLE) == BST_CHECKED;
					 return TRUE;
				case IDMUSICENABLE:
					 MusicToggle = IsDlgButtonChecked(hwndDlg, IDMUSICENABLE) == BST_CHECKED;
					 return TRUE;
				case IDMIDIDEV:
					 i = ComboBox_GetCurSel((HWND)lParam);
					 if (i != CB_ERR)
 					     MusicDevice = i;
					 return TRUE;
				case IDSTEREO:
					 settings.stereo = IsDlgButtonChecked(hwndDlg, IDSTEREO) == BST_CHECKED;
					 return TRUE;
				case IDVOICES:
					 i = ComboBox_GetCurSel((HWND)lParam);
					 if (i != CB_ERR)
 					     NumVoices = ivoices[i];
					 return TRUE;
				case IDSAMPLE:
					 i = ComboBox_GetCurSel((HWND)lParam);
					 if (i != CB_ERR)
 					     MixRate = isample[i];
					 return TRUE;
				case IDSOUND8BIT:
					 NumBits = 8;
					 return TRUE;
				case IDSOUND16BIT:
					 NumBits = 16;
					 return TRUE;
				case IDAMBENC:
					 settings.ambience = IsDlgButtonChecked(hwndDlg, IDAMBENC) == BST_CHECKED;
					 return TRUE;
				case IDDUKETK:
					 settings.duketalk = IsDlgButtonChecked(hwndDlg, IDDUKETK) == BST_CHECKED;
					 return TRUE;
				default: break;
		    }
		    break;
		default: break;
	}
	return FALSE;
}

static INT_PTR CALLBACK SavePageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDSDATA:
				{
					int i;
				    ResetForm(1<<TAB_GAME);
				    ResetForm(1<<TAB_MAPS);
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
  					    UserSavenum = i;
					return TRUE;
				}
				case IDSKILL:
				{
					int i;
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					    iSkill = i+1;
					return TRUE;
				}
				case IDSADULT:
					 settings.adultmode = IsDlgButtonChecked(hwndDlg, IDSADULT) == BST_CHECKED;
					 return TRUE;
				case IDSDETAL:
					 settings.highdetail = IsDlgButtonChecked(hwndDlg, IDSDETAL) == BST_CHECKED;
					 return TRUE;
				case IDSATAIM:
					 settings.autoaiming = IsDlgButtonChecked(hwndDlg, IDSATAIM) == BST_CHECKED;
					 return TRUE;
				case IDSMOUSE:
					 settings.invertmouse = IsDlgButtonChecked(hwndDlg, IDSMOUSE) == BST_CHECKED;
					 return TRUE;
				case IDSTILTS:
					 settings.tilting = IsDlgButtonChecked(hwndDlg, IDSTILTS) == BST_CHECKED;
					 return TRUE;
				case IDSMESGS:
					 settings.messages = IsDlgButtonChecked(hwndDlg, IDSMESGS) == BST_CHECKED;
					 return TRUE;
				default: break;
			}
			break;
		default: break;
	}
	return FALSE;
}

static INT_PTR CALLBACK GamePageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDGBUTTON:
				{
                     if (MenuGame[0] != 0)
                     {
                        getmapnames(MenuGame);
			            ResetForm(1<<TAB_MAPS);
			            ResetForm(1<<TAB_SAVE);
			         }
				     return TRUE;
				}
				case IDGDIRC:
				{
					int i;
					i = ComboBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
                       ComboBox_GetLBText((HWND)lParam, i, GameFolder);
                       strlwr(GameFolder);
                       if (i == 0)
                           GameFolder[0] = 0;
                       GetGameFiles(GameFolder);
                    }
				    ResetForm(1<<TAB_SAVE);
					return TRUE;
				}
				case IDGDATA:
				{
					int i;
				    ResetForm(1<<TAB_MAPS);
  					ResetForm(1<<TAB_SAVE);
  					ResetForm(99);
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
                       ListBox_GetText((HWND)lParam, i, MenuGame);
                       strlwr(MenuGame);
                    }
					return TRUE;
				}
				case IDGMAPS:
				{
					int i;
				    ResetForm(1<<TAB_MAPS);
  					ResetForm(1<<TAB_SAVE);
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					   {
					   ListBox_GetText((HWND)lParam, i, GameMap);
					   strlwr(GameMap);
					   iLevelNum = i;  // incase default maps
					   }
					return TRUE;
				}
				default: break;
			}
			break;
		default: break;
	}
	return FALSE;
}

static INT_PTR CALLBACK MapsPageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDMDIRC:
				{
					int i;
					i = ComboBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
                       ComboBox_GetLBText((HWND)lParam, i, MapsFolder);
                       strlwr(MapsFolder);
                       if (i == 0)
                           MapsFolder[0] = 0;
                       GetMapFiles(MapsFolder);
                    }
					return TRUE;
				}
				case IDMDATA:
				{
					int i;
				    ResetForm(1<<TAB_GAME);
				    ResetForm(1<<TAB_SAVE);
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
					    ListBox_GetText((HWND)lParam, i, MenuMap);
					    strlwr(MenuMap);
					}
					return TRUE;
				}
				case IDMFOLD:
				{
					int i;
					i = ComboBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
                       ComboBox_GetLBText((HWND)lParam, i, MusicFolder);
                       strlwr(MusicFolder);
                       if (i == 0)
                           MusicFolder[0] = 0;
                       GetMidiFiles(MusicFolder);
                    }
					return TRUE;
				}
				case IDMUSIC:
				{
					int i;
				    ResetForm(1<<TAB_SAVE);
					i = ListBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
					    ListBox_GetText((HWND)lParam, i, MenuMusic);
					    strlwr(MenuMusic);
					}
					return TRUE;
				}
				default: break;
			}
			break;
		default: break;
	}
	return FALSE;
}

static INT_PTR CALLBACK MultiPageProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    short i;
    HWND Hh, Hw;

	switch (uMsg)
	{
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDUNAME:
					Edit_GetText((HWND)lParam, PlayerNameArg, sizeof(PlayerNameArg));
                    //Edit_GetLine(hwnd, 0, PlayerNameArg, sizeof(PlayerNameArg));  // alternative
					return TRUE;
				case IDUIPAD:
					Edit_GetText((HWND)lParam, IPAddress, sizeof(IPAddress));
					return TRUE;
				case IDUIPPS:
				{
					int i;
					i = ComboBox_GetCurSel((HWND)lParam);
					if (i != CB_ERR)
					{
                        ComboBox_GetLBText((HWND)lParam, i, IPAddress);
 		                Hh = GetDlgItem(pages[TAB_MULT], IDUIPAD);
		                Edit_SetText(Hh, IPAddress);
                    }
					return TRUE;
				}
				case IDUHOST:
					 HostMulti = 1;
					 return TRUE;
				case IDUJOIN:
					 HostMulti = -1;
					 return TRUE;
				case IDUMASTER:
		             ResetForm(1<<TAB_MAPS);
			         ResetForm(1<<TAB_SAVE);
					 DedicatedArg = IsDlgButtonChecked(hwndDlg, IDUMASTER) == BST_CHECKED;
	                 Hh = GetDlgItem(pages[TAB_MULT], IDUNUMPLS);
	                 Hw = GetDlgItem(pages[TAB_MAPS], IDMDATA);
					 if (DedicatedArg == 0)
					 {
					     DedicatedArg = -1;
					     ComboBox_Enable(Hh, 0);
					     ListBox_Enable(Hw, 1);
					 }
					 else
					 {
					     ComboBox_Enable(Hh, 1);
					     ListBox_Enable(Hw, 0);
					 }
					 return TRUE;
				case IDUNUMPLS:
					 i = ComboBox_GetCurSel((HWND)lParam);
					 if (i != CB_ERR)
 					     NumPlayerArg = i+2;
					 return TRUE;
				case IDUFAKEMP:
		             ResetForm(1<<TAB_GAME);
			         ResetForm(1<<TAB_SAVE);
					 FakePlayer = IsDlgButtonChecked(hwndDlg, IDUFAKEMP) == BST_CHECKED;
	                 Hh = GetDlgItem(pages[TAB_MULT], IDUNUMBOT);
	                 Hw = GetDlgItem(pages[TAB_MULT], IDUSELECT);
					 if (FakePlayer == 0)
					 {
					     FakePlayer = -1;
					     ComboBox_Enable(Hh, 0);
					     Button_Enable(Hw, 1);
					 }
					 else
					 {
					     ComboBox_Enable(Hh, 1);
					     Button_Enable(Hw, 0);
					     Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUSELECT), FALSE);
					     SelectMulti = -1;
					 }
					 return TRUE;
				case IDUNUMBOT:
					 i = ComboBox_GetCurSel((HWND)lParam);
					 if (i != CB_ERR)
 					     NumBots = i+1;
					 return TRUE;
				case IDUSELECT:
				     ResetForm(1<<TAB_GAME);
				     ResetForm(1<<TAB_SAVE);
				     Hh = GetDlgItem(pages[TAB_MULT], IDUFAKEMP);
					 SelectMulti = IsDlgButtonChecked(hwndDlg, IDUSELECT) == BST_CHECKED;
					 if (SelectMulti == 0)
					 {
					     SelectMulti = -1;
					     Button_Enable(Hh, 1);
					 }
					 else
					 {
					     Button_Enable(Hh, 0);
					     Button_SetCheck(GetDlgItem(pages[TAB_MULT], IDUFAKEMP), FALSE);
					     FakePlayer = -1;
					 }
 				     ResetForm(1<<TAB_GAME);
				     ResetForm(1<<TAB_SAVE);
					 return TRUE;
				default: break;
			}
			break;
		default: break;
	}
	return FALSE;
}

static void SetPage(int n)
{
	HWND tab;
	int cur;

	tab = GetDlgItem(startupdlg, WIN_STARTWIN_TABCTL);
	cur = (int)SendMessage(tab, TCM_GETCURSEL,0,0);
	ShowWindow(pages[cur],SW_HIDE);
	SendMessage(tab, TCM_SETCURSEL, n, 0);
	ShowWindow(pages[n],SW_SHOW);
	mode = n;

	SetFocus(GetDlgItem(startupdlg, WIN_STARTWIN_TABCTL));
}

static void EnableConfig(int n)
{
	EnableWindow(GetDlgItem(startupdlg, WIN_STARTWIN_START), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDVIDMODE), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDFULLSCREEN), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDWIDESCREEN), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDPCACHE), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDTCACHE), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDHTILES), n);
	EnableWindow(GetDlgItem(pages[TAB_VIDEO], IDHMODEL), n);
	EnableWindow(GetDlgItem(pages[TAB_SOUND], IDVOICES), n);
	EnableWindow(GetDlgItem(pages[TAB_SOUND], IDSAMPLE), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDGDIRC), n);
	EnableWindow(GetDlgItem(pages[TAB_GAME], IDGDATA), n);
	EnableWindow(GetDlgItem(pages[TAB_GAME], IDGMAPS), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDMDATA), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDMDIRC), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDSKILL), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDMFOLD), n);
	EnableWindow(GetDlgItem(pages[TAB_MAPS], IDMUSIC), n);
	EnableWindow(GetDlgItem(pages[TAB_SAVE], IDSDATA), n);
	EnableWindow(GetDlgItem(pages[TAB_MULT], IDUNUMPLS), n);
}

static INT_PTR CALLBACK startup_dlgproc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static HBITMAP hbmp = NULL;
	HDC hdc;

	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			HWND hwnd;
			RECT r, rdlg, chrome, rtab, rcancel, rstart, rdes;
			int xoffset = 0, yoffset = 0;

			// Fetch the positions (in screen coordinates) of all the windows we need to tweak
			ZeroMemory(&chrome, sizeof(chrome));
			AdjustWindowRect(&chrome, GetWindowLong(hwndDlg, GWL_STYLE), FALSE);
			GetWindowRect(hwndDlg, &rdlg);
			GetWindowRect(GetDlgItem(hwndDlg, WIN_STARTWIN_TABCTL), &rtab);
			GetWindowRect(GetDlgItem(hwndDlg, WIN_STARTWIN_CANCEL), &rcancel);
			GetWindowRect(GetDlgItem(hwndDlg, WIN_STARTWIN_START), &rstart);
			GetWindowRect(GetDlgItem(hwndDlg, WIN_STARTWIN_ABORT), &rdes);

			// Knock off the non-client area of the main dialogue to give just the client area
			rdlg.left -= chrome.left; rdlg.top -= chrome.top;
			rdlg.right -= chrome.right; rdlg.bottom -= chrome.bottom;

			// Translate them to client-relative coordinates wrt the main dialogue window
			rtab.right -= rtab.left - 1; rtab.bottom -= rtab.top - 1;
			rtab.left  -= rdlg.left; rtab.top -= rdlg.top;

			rcancel.right -= rcancel.left - 1; rcancel.bottom -= rcancel.top - 1;
			rcancel.left -= rdlg.left; rcancel.top -= rdlg.top;

			rstart.right -= rstart.left - 1; rstart.bottom -= rstart.top - 1;
			rstart.left -= rdlg.left; rstart.top -= rdlg.top;

			rdes.right -= rdes.left - 1; rdes.bottom -= rdes.top - 1;
			rdes.left -= rdlg.left; rdes.top -= rdlg.top;

			// And then convert the main dialogue coordinates to just width/length
			rdlg.right -= rdlg.left - 1; rdlg.bottom -= rdlg.top - 1;
			rdlg.left = 0; rdlg.top = 0;

			// Load the bitmap into the bitmap control and fetch its dimensions
			hbmp = LoadBitmap((HINSTANCE)win_gethinstance(), MAKEINTRESOURCE(RSRC_BMP));
			hwnd = GetDlgItem(hwndDlg,WIN_STARTWIN_BITMAP);
			SendMessage(hwnd, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)hbmp);
			GetClientRect(hwnd, &r);
			xoffset = r.right;
			yoffset = r.bottom - rdlg.bottom;

			// Shift and resize the controls that require it
			rtab.left += xoffset; rtab.bottom += yoffset;
			rcancel.left += xoffset; rcancel.top += yoffset;
			rstart.left += xoffset; rstart.top += yoffset;
			rdes.left += xoffset; rdes.top += yoffset;
			rdlg.right += xoffset;
			rdlg.bottom += yoffset;

			// Move the controls to their new positions
			MoveWindow(GetDlgItem(hwndDlg, WIN_STARTWIN_TABCTL), rtab.left, rtab.top, rtab.right, rtab.bottom, FALSE);
			MoveWindow(GetDlgItem(hwndDlg, WIN_STARTWIN_CANCEL), rcancel.left, rcancel.top, rcancel.right, rcancel.bottom, FALSE);
			MoveWindow(GetDlgItem(hwndDlg, WIN_STARTWIN_START), rstart.left, rstart.top, rstart.right, rstart.bottom, FALSE);
			MoveWindow(GetDlgItem(hwndDlg, WIN_STARTWIN_ABORT), rdes.left, rdes.top, rdes.right, rdes.bottom, FALSE);

			// Move the main dialogue to the centre of the screen
			hdc = GetDC(NULL);
			rdlg.left = (GetDeviceCaps(hdc, HORZRES) - rdlg.right) / 2;
			rdlg.top = (GetDeviceCaps(hdc, VERTRES) - rdlg.bottom) / 2;
			ReleaseDC(NULL, hdc);
			MoveWindow(hwndDlg, rdlg.left + chrome.left, rdlg.top + chrome.left,
				rdlg.right + (-chrome.left+chrome.right), rdlg.bottom + (-chrome.top+chrome.bottom), TRUE);

			// Add tabs to the tab control
			{
				TCITEM tab;

				hwnd = GetDlgItem(hwndDlg, WIN_STARTWIN_TABCTL);

				ZeroMemory(&tab, sizeof(tab));
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Video");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_VIDEO, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Sound");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_SOUND, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Addons");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_GAME, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("User Maps");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_MAPS, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Gaming");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_SAVE, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Multiplay");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_MULT, (LPARAM)&tab);
				tab.mask = TCIF_TEXT;
				tab.pszText = TEXT("Logs");
				SendMessage(hwnd, TCM_INSERTITEM, (WPARAM)TAB_MESG, (LPARAM)&tab);

				// Work out the position and size of the area inside the tab control for the pages
				ZeroMemory(&r, sizeof(r));
				GetClientRect(hwnd, &r);
				SendMessage(hwnd, TCM_ADJUSTRECT, FALSE, (LPARAM)&r);
				r.right -= r.left-1;
				r.bottom -= r.top-1;
				r.top += rtab.top;
				r.left += rtab.left;

				// Create the pages and position them in the tab control, but hide them
				pages[TAB_VIDEO] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_VIDEO), hwndDlg, VideoPageProc);
				pages[TAB_SOUND] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_SOUND), hwndDlg, SoundPageProc);
				pages[TAB_SAVE] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_SAVE), hwndDlg, SavePageProc);
				pages[TAB_GAME] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_GAME), hwndDlg, GamePageProc);
				pages[TAB_MAPS] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_MAPS), hwndDlg, MapsPageProc);
				pages[TAB_MULT] = CreateDialog((HINSTANCE)win_gethinstance(),
					              MAKEINTRESOURCE(WIN_STARTWINPAGE_MULT), hwndDlg, MultiPageProc);
				pages[TAB_MESG] = GetDlgItem(hwndDlg, WIN_STARTWIN_MESSAGES);
				SetWindowPos(pages[TAB_VIDEO], hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_SOUND], hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_SAVE],  hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_GAME],  hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_MAPS],  hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_MULT],  hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);
				SetWindowPos(pages[TAB_MESG],  hwnd,r.left,r.top,r.right,r.bottom,SWP_HIDEWINDOW);

				// Tell the editfield acting as the console to exclude the width of the scrollbar
				GetClientRect(pages[TAB_MESG],&r);
				r.right -= GetSystemMetrics(SM_CXVSCROLL)+4;
				r.left = r.top = 0;
				SendMessage(pages[TAB_MESG], EM_SETRECTNP,0,(LPARAM)&r);

				// Set a tab stop in the game data listbox
				{
				DWORD tabs[1] = { 150 };
				ListBox_SetTabStops(GetDlgItem(pages[TAB_SAVE], IDSDATA), 1, tabs);
				ListBox_SetTabStops(GetDlgItem(pages[TAB_GAME], IDGDATA), 1, tabs);
				ListBox_SetTabStops(GetDlgItem(pages[TAB_MAPS], IDMDATA), 1, tabs);
				ListBox_SetTabStops(GetDlgItem(pages[TAB_MAPS], IDMUSIC), 1, tabs);
				ListBox_SetTabStops(GetDlgItem(pages[TAB_MAPS], IDSKILL), 1, tabs);
				}

				SetFocus(GetDlgItem(hwndDlg, WIN_STARTWIN_START));
				SetWindowText(hwndDlg, apptitle);
			}
			return FALSE;
		}

		case WM_NOTIFY:
		{
			LPNMHDR nmhdr = (LPNMHDR)lParam;
			int cur;
			if (nmhdr->idFrom != WIN_STARTWIN_TABCTL) break;
			cur = (int)SendMessage(nmhdr->hwndFrom, TCM_GETCURSEL,0,0);
			switch (nmhdr->code)
			{
				case TCN_SELCHANGING:
				{
					if (cur < 0 || !pages[cur]) break;
					ShowWindow(pages[cur],SW_HIDE);
					return TRUE;
				}
				case TCN_SELCHANGE:
				{
					if (cur < 0 || !pages[cur]) break;
					ShowWindow(pages[cur],SW_SHOW);
					return TRUE;
				}
			}
			break;
		}

		case WM_CLOSE:
			if (mode == TAB_VIDEO)
			    done = 0;
			else
			    quitevent++;
			return TRUE;

		case WM_DESTROY:
			if (hbmp)
			{
				DeleteObject(hbmp);
				hbmp = NULL;
			}

			if (pages[TAB_SAVE])
			{
				DestroyWindow(pages[TAB_SAVE]);
				pages[TAB_SAVE] = NULL;
			}

			if (pages[TAB_GAME])
			{
				DestroyWindow(pages[TAB_GAME]);
				pages[TAB_GAME] = NULL;
			}

			if (pages[TAB_MAPS])
			{
				DestroyWindow(pages[TAB_MAPS]);
				pages[TAB_MAPS] = NULL;
			}

			if (pages[TAB_VIDEO])
			{
				DestroyWindow(pages[TAB_VIDEO]);
				pages[TAB_VIDEO] = NULL;
			}

			if (pages[TAB_SOUND])
			{
				DestroyWindow(pages[TAB_SOUND]);
				pages[TAB_SOUND] = NULL;
			}

			startupdlg = NULL;
			return TRUE;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			    case WIN_STARTWIN_ABORT:
			         ResetForm(1<<TAB_VIDEO);
			         ResetForm(1<<TAB_SOUND);
			         ResetForm(1<<TAB_SAVE);
			         ResetForm(1<<TAB_GAME);
			         ResetForm(1<<TAB_MAPS);
			         ResetForm(1<<TAB_MULT);
			         return TRUE;
				case WIN_STARTWIN_CANCEL:
					 if (mode == TAB_VIDEO)
					     done = 0;
					 else
					     quitevent++;
					 return TRUE;
				case WIN_STARTWIN_START:
				     done = 1;
				     return TRUE;
			}
			return FALSE;

		case WM_CTLCOLORSTATIC:
			if ((HWND)lParam == pages[TAB_MESG])
				return (BOOL)GetSysColorBrush(COLOR_WINDOW);
			break;

		default: break;
	}

	return FALSE;
}

int startwin_open(void)
{
	INITCOMMONCONTROLSEX icc;
    int i = CONFIG_ReadSetup();
    char *cmdbuf = NULL;
    int j;

    cmdbuf = strdup(GetCommandLine());

	if (startupdlg)
	    return 1;

	if (PlayerNameArg[0] == 0)
	    Bstrcpy(PlayerNameArg, "Player");
	if (IPAddressArg[0] == 0)
	    Bstrcpy(IPAddressArg[0], "127.0.0.0");

	icc.dwSize = sizeof(icc);
	icc.dwICC = ICC_TAB_CLASSES;
	InitCommonControlsEx(&icc);

    j = 1;
  	if (strchr(cmdbuf, '-') == 0 && strchr(cmdbuf, '/') == 0)        // check for parameters
  	    j = 0;

    if (strstr(cmdbuf, "setup"))
    {
       ForceSetup = 1;
       settings.forcesetup = 1;
       j = 0;
    }

	if (ForceSetup == 1 && j == 0)
    {
   	   startupdlg = CreateDialog((HINSTANCE)win_gethinstance(), MAKEINTRESOURCE(WIN_STARTWIN), NULL, startup_dlgproc);
	   if (startupdlg)
       {
		  SetPage(TAB_MESG);
		  EnableConfig(0);
		  return 0;
	   }
	}

	return -1;
}

int startwin_close(void)
{
	if (!startupdlg)
	    return 1;
	DestroyWindow(startupdlg);
	startupdlg = NULL;
	return 0;
}

int startwin_puts(const char *buf)
{
	const char *p = NULL, *q = NULL;
	char workbuf[1024];
	static int newline = 0;
	int curlen, linesbefore, linesafter;
	HWND edctl;
	int vis;

	if (!startupdlg)
	    return 1;

	edctl = pages[TAB_MESG];
	if (!edctl)
	    return -1;

	vis = ((int)SendMessage(GetDlgItem(startupdlg, WIN_STARTWIN_TABCTL), TCM_GETCURSEL,0,0) == TAB_MESG);

	if (vis)
	    SendMessage(edctl, WM_SETREDRAW, FALSE,0);
	curlen = SendMessage(edctl, WM_GETTEXTLENGTH, 0,0);
	SendMessage(edctl, EM_SETSEL, (WPARAM)curlen, (LPARAM)curlen);
	linesbefore = SendMessage(edctl, EM_GETLINECOUNT, 0,0);
	p = buf;
	while (*p)
	{
		if (newline)
		{
			SendMessage(edctl, EM_REPLACESEL, 0, (LPARAM)"\r\n");
			newline = 0;
		}
		q = p;
		while (*q && *q != '\n') q++;
		memcpy(workbuf, p, q-p);
		if (*q == '\n')
		{
			if (!q[1])
			{
				newline = 1;
				workbuf[q-p] = 0;
			}
			else
			{
				workbuf[q-p] = '\r';
				workbuf[q-p+1] = '\n';
				workbuf[q-p+2] = 0;
			}
			p = q+1;
		}
		else
		{
			workbuf[q-p] = 0;
			p = q;
		}
		SendMessage(edctl, EM_REPLACESEL, 0, (LPARAM)workbuf);
	}
	linesafter = SendMessage(edctl, EM_GETLINECOUNT, 0,0);
	SendMessage(edctl, EM_LINESCROLL, 0, linesafter-linesbefore);
	if (vis)
	    SendMessage(edctl, WM_SETREDRAW, TRUE,0);
	return 0;
}

int startwin_settitle(const char *str)
{
	if (!startupdlg)
	    return 1;
	SetWindowText(startupdlg, str);
	return 0;
}

int startwin_idle(void *v)
{
	if (!startupdlg || !IsWindow(startupdlg))
	    return 0;
	if (IsDialogMessage(startupdlg, (MSG*)v))
	    return 1;
	return 0;
}

extern char *duke3dgrp;

int startwin_run(void)
{
	HWND hwnd;
	MSG msg;
	long fl;

	if (!startupdlg)
	    return 1;

	done = -1;

    GetSavedGames();

#ifdef JFAUD
    EnumAudioDevs(&wavedevs, NULL, NULL);
#endif

	SetPage(TAB_MESG);
	EnableConfig(1);

	settings.fullscreen = ScreenMode;
	settings.xdim = ScreenWidth;
	settings.ydim = ScreenHeight;
	settings.bpp = ScreenBPP;
	settings.forcesetup = ForceSetup;
	settings.usemouse = UseMouse;
	settings.usejoy = UseJoystick;
	settings.game = gametype;
	strncpy(settings.selectedgrp, duke3dgrp, BMAX_PATH);
	settings.widescreen = glwidescreen;
	settings.brightness = ud.brightness;
	settings.stereo = ReverseStereo;

	settings.adultmode   = !ud.lockout;
	settings.highdetail  = ud.detail;
	settings.autoaiming  = AutoAim;
	settings.invertmouse = ud.mouseflip;
	settings.tilting     = ud.screen_tilting;
	settings.messages    = ud.fta_on;

    settings.ambience = AmbienceToggle;
    settings.duketalk = VoiceToggle;

    PreCache = useprecache;
    TexCache = glusetexcache;
    Tiles = usehightile;
    Model = usemodels;
    TexComp = glusetexcompr;
    LCD = LcdMon;

	PopulateForm(-1);

    MenuMap[0] = 0;
    MenuGame[0] = 0;
    UserSavenum = -1;

	SetPage(TAB_VIDEO);

	while (done < 0)
	{
		switch (GetMessage(&msg, NULL, 0,0))
		{
			case 0: done = 1; break;
			case -1: return -1;
			default:
				 if (IsWindow(startupdlg) && IsDialogMessage(startupdlg, &msg))
				     break;
				 TranslateMessage(&msg);
				 DispatchMessage(&msg);
				 break;
		}
	}

	SetPage(TAB_MESG);
	EnableConfig(0);

    hwnd = GetDlgItem(pages[TAB_VIDEO], IDBRIGHT);
    settings.brightness = SendMessage(hwnd, TBM_GETPOS, 0, 0);

	if (done)
	{
		ScreenMode = settings.fullscreen;
		ScreenWidth = settings.xdim;
		ScreenHeight = settings.ydim;
		ScreenBPP = settings.bpp;
		ForceSetup = settings.forcesetup;
		UseMouse = settings.usemouse;
		UseJoystick = settings.usejoy;
		duke3dgrp = settings.selectedgrp;
		gametype = settings.game;

        ReverseStereo = settings.stereo;
		glwidescreen = settings.widescreen;
        ud.brightness = settings.brightness;

	    ud.lockout        = !settings.adultmode;
	    ud.detail         = settings.highdetail;
	    AutoAim           = settings.autoaiming;
	    ud.mouseflip      = settings.invertmouse;
	    ud.screen_tilting = settings.tilting;
	    ud.fta_on         = settings.messages;

        useprecache = PreCache;
        glusetexcache = TexCache;
        usehightile = Tiles;
        usemodels = Model;
        glusetexcompr = TexComp;
        LcdMon = LCD;

        AmbienceToggle = settings.ambience;
        VoiceToggle = settings.duketalk;

        if (SelectMulti || FakePlayer)
        {
        hwnd = GetDlgItem(pages[TAB_MULT], IDUNAME);
        Edit_GetText(hwnd, PlayerNameArg, sizeof(PlayerNameArg));
        }
	}

    CheckIPAddress();               // add to list if not there  wxsm

	if (wavedevs)
	{
		struct audioenumdev *d, *e;
		free(wavedevs->drvs);
		for (e=wavedevs->devs; e; e=d)
		{
			d = e->next;
			if (e->devs)
			    free(e->devs);
		 	free(e);
		}
		free(wavedevs);
	}

//    fl = kopen4load("Duke3dw.cfg",0);
//    if (fl == -1)
//        CONFIG_WriteSetup();    // temp
//    kclose(fl);

	return done;
}

//-----------------------------------------------------------------------------

void PopulateSound(void)
{
    HWND hwnd;
	UINT nummididevs;
	MIDIOUTCAPS mididev;
	UINT i,j;

	nummididevs = midiOutGetNumDevs();
    hwnd = GetDlgItem(pages[TAB_SOUND], IDMIDIDEV);
	ComboBox_ResetContent(hwnd);

	for (i=0; i<nummididevs; i++)
	{
		midiOutGetDevCaps(i, &mididev, sizeof(MIDIOUTCAPS));
		ComboBox_AddString(hwnd, mididev.szPname);
	}

	if (MusicDevice < 0)
	{
		EnableWindow(GetDlgItem(pages[TAB_SOUND], IDMIDIDEV), FALSE);
		ComboBox_SetCurSel(GetDlgItem(pages[TAB_SOUND], IDMIDIDEV),0);
		MusicToggle = 0;
	}
	else
	{
		if (MusicDevice >= nummididevs)
		    MusicDevice = nummididevs-1;
		EnableWindow(GetDlgItem(pages[TAB_SOUND], IDMIDIDEV), TRUE);
		ComboBox_SetCurSel(GetDlgItem(pages[TAB_SOUND], IDMIDIDEV), MusicDevice);

        hwnd = GetDlgItem(pages[TAB_SOUND], IDMIDIDEV);
        ComboBox_SetCurSel(hwnd, MusicDevice);
	}

	if (MusicToggle < 1)
	{
		Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDMUSICENABLE), FALSE);
		MusicToggle = 0;
	}
	else
	{
		Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDMUSICENABLE), TRUE);
		MusicToggle = 1;
	}

	if (SoundToggle < 1)
	{
		Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSOUNDENABLE), FALSE);
		SoundToggle = 0;
	}
	else
	{
		Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSOUNDENABLE), TRUE);
		SoundToggle = 1;
	}

    {
    hwnd = GetDlgItem(pages[TAB_SOUND], IDVOICES);

    for (i=0; i<8; i++)
         ComboBox_AddString(hwnd, Voices[i]);

    j = 5;
    for (i=0; i<8; i++)
    {
        if (ivoices[i] == NumVoices)
        {
           j = i;
           break;
        }
    }
    ComboBox_SetCurSel(hwnd, j);

    hwnd = GetDlgItem(pages[TAB_SOUND], IDSAMPLE);

    for (i=0; i<7; i++)
         ComboBox_AddItemData(hwnd, Sample[i]);

    j = 3;
    for (i=0; i<7; i++)
        {
        if (isample[i] == MixRate)
           {
           j = i;
           break;
           }
        }
    ComboBox_SetCurSel(hwnd, j);
    }

	if (NumBits == 8)
        Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSOUND8BIT), TRUE);
	else
	    Button_SetCheck(GetDlgItem(pages[TAB_SOUND], IDSOUND16BIT), TRUE);
}

void SetDefGames(void)
{
    HWND hwnd;
    short i;

    hwnd = GetDlgItem(pages[TAB_GAME], IDGMAPS);
	ListBox_ResetContent(hwnd);

    for (i=0; i<44; i++)
         ListBox_AddString(hwnd, defgames[i]);

    ListBox_SetCurSel(hwnd, -1);
    iLevelNum = -1;

    if (shareware)
        ListBox_Enable(hwnd, 0);
}

void getmapnames(char *fname)
{
    HWND hwnd;
    FILE *fn;
    char grpname[80];
    char gbuf[16];
    int i, j, x, z;

    hwnd = GetDlgItem(pages[TAB_GAME], IDGMAPS);
	ListBox_ResetContent(hwnd);

	if (Bstrcmp(fname, ".grp") == 0)
	    return;

    if (Bstrcmp(fname, "Duke3d.grp") == 0)
       {
       SetDefGames();
       return;
       }
    else
       Bsprintf(grpname, "Games\\%s", fname);

    fn = fopen(grpname,"rb");
    x = 0;
    if (fn)
    {
	   for (i=0; i<1000; i++)  // 1000 possible entries max
       {
           fread((gbuf),(16),(1),(fn));
           for (j=12; j<16; j++)
                gbuf[j] = 0;
           if (gbuf[0] < 0x21 || gbuf[0] > 0x7a)
               break;
           if (i == 0)
           {
              if (!Bstrcmp(grpname, "KenSilverman"))
                  break;
           }
           else
           {
              strlwr(gbuf);
              if (gbuf[0] > 0x60)
                  gbuf[0] = gbuf[0] ^ 0x20 ;
              j = strlen(gbuf)-4;
		      if (Bstrncasecmp(gbuf+j, ".map", 4) == 0)
              {
                 ListBox_AddString(hwnd, gbuf);
                 x++;
              }
           }
       }
       fclose(fn);
       ListBox_SetCurSel(hwnd, -1);
	}
}

void GetGameFiles(char *gpath)
{
    HWND hwnd;
    char gbuf[128];
    int x;

    x = Bstrcasecmp(GameFolder, LastGameFolder);
    if (x == 0)
        return;
    strcpy(LastGameFolder, gpath);

    hwnd = GetDlgItem(pages[TAB_GAME], IDGDATA);
	ListBox_ResetContent(hwnd);

    clearfilenames();
    Bsprintf(gbuf, "%s/%s", RootFolder, gpath);
    getfiles(gbuf,"*.grp");
    x = 0;
    while (findfiles)
    {
          Bsprintf(gbuf,"%s",findfiles->name);
          if (strlen(gbuf) > 4)
          {
             strlwr(gbuf);
             if (gbuf[0] > 0x60)
                 gbuf[0] = gbuf[0] ^ 0x20 ;
             ListBox_AddString(hwnd, gbuf);
             x++;
          }
          if (x >= 200)
              break;
          findfiles = findfiles->next;
    }

    clearfilenames();
    Bsprintf(gbuf, "%s/%s/", RootFolder, gpath);
    getfiles(gbuf,"*.zip");
    while (findfiles)
    {
          Bsprintf(gbuf,"%s",findfiles->name);
          if (!strstr(gbuf, "_hrp"))
          {
             strlwr(gbuf);
             if (gbuf[0] > 0x60)
                 gbuf[0] = gbuf[0] ^ 0x20 ;
             ListBox_AddString(hwnd, gbuf);
             x++;
          }
          if (x >= 200)
              break;
          findfiles = findfiles->next;
    }
    ListBox_SetCurSel(hwnd, -1);
    if (shareware)
        ListBox_Enable(hwnd, 0);
    ResetForm(99);
}

void GetMapFiles(char *mpath)
{
    HWND hwnd;
    char mbuf[128];
    int x;

    hwnd = GetDlgItem(pages[TAB_MAPS], IDMDATA);
	ListBox_ResetContent(hwnd);

    clearfilenames();
    Bsprintf(mbuf, "%s/%s/", RootFolder, mpath);
    getfiles(mbuf,"*.map");
    x = 0;
    while (findfiles)
    {
          Bsprintf(mbuf,"%s",findfiles->name);
          strlwr(mbuf);
          if (mbuf[0] > 0x60)
              mbuf[0] = mbuf[0] ^ 0x20 ;
          ListBox_AddString(hwnd, mbuf);
          x++;
          if (x >= 500)
              break;
          findfiles = findfiles->next;
    }
    ListBox_SetCurSel(hwnd, -1);
    if (shareware)
        ListBox_Enable(hwnd, 0);
}

void GetMidiFiles(char *mpath)
{
    HWND hwnd;
    char mbuf[128];
    int x;

    hwnd = GetDlgItem(pages[TAB_MAPS], IDMUSIC);
	ListBox_ResetContent(hwnd);

    clearfilenames();
    Bsprintf(mbuf, "%s/%s/", RootFolder, mpath);
    getfiles(mbuf,"*.mid");
    x = 0;
    while (findfiles)
    {
          Bsprintf(mbuf,"%s",findfiles->name);
          strlwr(mbuf);
          if (mbuf[0] > 0x60)
              mbuf[0] = mbuf[0] ^ 0x20 ;
          ListBox_AddString(hwnd, mbuf);
          x++;
          if (x >= 500)
              break;
          findfiles = findfiles->next;
    }
    ListBox_SetCurSel(hwnd, -1);
    if (shareware)
        ListBox_Enable(hwnd, 0);
}

//-----------------------------------------------------------------------------

int getdirs(char *path)
{
	CACHE1D_FIND_REC *r;

	clearfilenames();
	finddirs = klistpath(path,"*",CACHE1D_FIND_DIR);
	finddirshigh = finddirs;
	return(0);
}

int getfiles(char *path, char kind[])
{
	CACHE1D_FIND_REC *r;

	clearfilenames();
    findfiles = klistpath(path,kind,CACHE1D_FIND_FILE);
	findfileshigh = findfiles;
	return(0);
}

void LoadGameDes(short sv_num, char *desc)
{
    int i;
    long fl = 0;
    char sgame[64];
    char xbuf[20];

    sprintf(sgame,"game%d.sav",sv_num);
    fl = fopen(sgame,"rb");
    fseek(fl, 26, 0);
    fread((xbuf),(19),(1),(fl));
    for (i=0; i<20; i++)
    {
        if (xbuf[i] < 0x20 || xbuf[i] > 0x7a)
            xbuf[i] = 0;
    }
    strcpy(desc, "        ");
    strcat(desc, xbuf);
    fclose(fl);
}

void GetSavedGames(void)
{
    short x = 0;
    char sbuf[128];

    clearfilenames();
    strcpy(tempbuf,"/");
    getfiles(tempbuf,"*.sav");
    x = 0;
    while (findfiles)
    {
          Bsprintf(tempbuf,"%s",findfiles->name);
          if (strlen(tempbuf) < 24)
          {
             sbuf[0] = 0;
             LoadGameDes(x, &sbuf);
             strcat(tempbuf, sbuf);
             strlwr(tempbuf);
             if (tempbuf[0] > 0x60)
                 tempbuf[0] = tempbuf[0] ^ 0x20 ;
             strcpy(UserSaves[x], tempbuf);
             x++;
          }
          if (x >= 10)
              break;
          findfiles = findfiles->next;
    }
    NumUserSaves = x;
}

void CheckIPAddress(void)
{
    short i;

    for (i=0; i<8; i++)
    {
        if (strlen(IPAddressArg[i]) < 8)
            break;
        if (Bstrcasecmp(IPAddressArg[i], IPAddress) == 0)
        {
            i = -1;
            break;
        }
    }

    if (i != -1)
    {
       if (i > 6)
           i = 7;
       strcpy(IPAddressArg[i], IPAddress);
    }
}

